﻿using System;
using System.Globalization;
using Microsoft.Data.Schema.Build;
using Microsoft.Data.Schema.Common;
using Microsoft.Data.Schema.Extensibility;
using Microsoft.Data.Schema;

namespace MySql.Dsp.Build
{
    /// <summary>
    /// The DatabaseTaskHost implementation represents the point in the design
    /// where command-line build and interactive (VS) project interaction
    /// converge. 
    /// </summary>
    internal class MySqlTaskHost : DatabaseTaskHost
    {
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="dsp"></param>
        /// <param name="errors"></param>
        /// <param name="ideMode"></param>
        public MySqlTaskHost(DatabaseSchemaProvider dsp, ErrorManager errors, bool ideMode)
            : base(dsp, errors, ideMode, null)
        {
            // Intentionally left blank
        }

        /// <summary>
        /// Returns default values for property. or string.Empty.
        /// </summary>
        /// <param name="propertyName"></param>
        /// <returns></returns>
        public override string GetDefaultValue(string propertyName, DatabaseTaskHost.PropertySource source)
        {
            string defaultValue = string.Empty;

            //switch (propertyName)
            //{
            //    case SqlCeProjectProperties.DSP:
            //        return SqlCeProjectProperties.Default_DSP;
            //    case SqlCeProjectProperties.LCID:
            //        return SqlCeProjectProperties.Default_LCID;
            //    case SqlCeProjectProperties.OutputType:
            //        return SqlCeProjectProperties.OutputTypeValues.Script;
            //    case SqlCeProjectProperties.SuppressWarnings:
            //        return SqlCeProjectProperties.Default_SuppressWarnings;
            //    case SqlCeProjectProperties.TreatWarningsAsErrors:
            //        return SqlCeProjectProperties.Default_TreatWarningsAsErrors;
            //}
            return defaultValue;
        }

        /// <summary>
        /// Intializes with DataSchemaModelConstructor
        /// </summary>
        /// <param name="smc"></param>
        protected override void InitializeConstructor(Microsoft.Data.Schema.SchemaModel.DataSchemaModelConstructor smc)
        {
            base.InitializeConstructor(smc);
        }

        protected override bool SetValue(string propertyName, object value, PropertySource source, out bool defaultUsed)
        {
            defaultUsed = false;
            switch (propertyName)
            {
                //case SqlCeProjectProperties.OutputType:
                //    {
                //        string valueAsString = value.ToString();
                //        if (valueAsString.Equals(SqlCeProjectProperties.OutputTypeValues.Script, StringComparison.OrdinalIgnoreCase) ||
                //            valueAsString.Equals(SqlCeProjectProperties.OutputTypeValues.Database, StringComparison.OrdinalIgnoreCase) ||
                //            valueAsString.Equals(SqlCeProjectProperties.OutputTypeValues.ScriptAndDatabase, StringComparison.OrdinalIgnoreCase))
                //        {
                //            return true;
                //        }
                //        AddValidationError(SqlCeProjectProperties.OutputType, valueAsString, SqlCeProjectProperties.OutputTypeValues.Script);
                //        defaultUsed = true;
                //        return false;
                //    }
                //case SqlCeProjectProperties.LCID:
                //    {
                //        CultureInfo[] cultures = CultureInfo.GetCultures(CultureTypes.AllCultures);
                //        string LcidString = value.ToString();
                //        foreach (CultureInfo culture in cultures)
                //        {
                //            if (string.Compare(LcidString, culture.LCID.ToString(), StringComparison.OrdinalIgnoreCase) == 0)
                //            {
                //                return true;
                //            }
                //        }
                //        AddValidationError(SqlCeProjectProperties.LCID, LcidString, SqlCeProjectProperties.Default_LCID);
                //        defaultUsed = true;
                //        return false;
                //    }
            }
            return base.SetValue(propertyName, value, source, out defaultUsed);
        }

        private void AddValidationError(string propertyName, string valueAsString, string defaultValue)
        {
            string errorMessage = "";
            //string errorMessage = string.Format(
            //    CultureInfo.CurrentCulture,
            //    Resources.PropertyInvalidDefaultUsed,
            //    propertyName,
            //    valueAsString,
            //    defaultValue);
            this.Errors.Add(new DataSchemaError(errorMessage, null, ErrorSeverity.Warning), ErrorManager.DeployCategory);
        }

        protected override bool MatchesDataSchemaModelValue(string propertyName, object value, PropertySource source)
        {
            return true;
        }

        protected override void VerifyConfigurationAfterSet(string propertyName, object value, PropertySource source)
        {
        }

        #region Constructor 

        internal class Constructor : DatabaseTaskHostConstructor
        {
            protected override DatabaseTaskHost ConstructServiceImplementation()
            {
                MySqlTaskHost host = new MySqlTaskHost(this.DatabaseSchemaProvider, this.Errors, this.IdeMode);
                return host;
            }
        }

        #endregion

    }
}
